var util = require('util');
var path = require('path');
var wizzi = require('../wizzi');
var log = require('../util/log')(module, { tofile: true });
var verify = wizzi.verify;
var JsWizziContext = require('../esprima/jswizzicontext').context;
var interpolate = require('./interpolate');
var types = require('../util/types');
var work = {};
work.lineSep = "__LS__";
work.textSep = "__TS__";
var md = module.exports = {};
md.getParamValues = function (params, args, node) {
    // console.log('getParamValues', params, args);
    var _paramArray = [], _argsArray = [];
    if (params !== null && typeof params === 'string' && params.length > 0) {
        _paramArray = params.trim().split(',');
    }
    if (args !== null && typeof args === 'string' && args.length > 0) {
        _argsArray = args.trim().split(',');
    }
    var result = [];
    for (var i = 0; i < _paramArray.length; i++) {
        var name, 
            type = 'string', 
            defaultValue = null, 
            value = null,
            hasparamvalue = false,
            hasdefaultvalue = false;
        var param_parts = _paramArray[i].trim().split(/[|]/); // default value separator
        if (param_parts.length > 2) throw wizzi.error.NodeError('Argument ' + i + ' is malformed. Expected name[:type[|default]]', node);
        if (param_parts.length == 2) {
            defaultValue = param_parts[1].trim();
            hasdefaultvalue = true;
        }
        var param_parts = param_parts[0].trim().split(/[:]/); // type separator
        if (param_parts.length > 2) throw wizzi.errors.NodeError('Argument ' + i + ' is malformed. Expected name[:type[|default]]', node);
        if (param_parts.length == 2) type = param_parts[1];
        name = param_parts[0];
        if (name.substr(0, 1) === '&') {
            name = name.substr(1);
            type = 'object';
        }
        if (_argsArray.length > i)
        {
            value = _argsArray[i].trim();
            if (type === 'object') {
                if (value.substr(0, 1) !== '&') {
                    throw wizzi.errors.NodeError('Error getParamValues. Argument (' + i + ') must be an object reference. Found ' + value, node);
                } else {
                    value = value.substr(1);
                }
            } else {
                value = types.objectify(_argsArray[i].trim(), type, node, true);
            }
            hasparamvalue = true;
        }
        else if (hasdefaultvalue) {
            value = types.objectify(defaultValue, type, node, true);
        }
        else
            throw wizzi.errors.NodeError('Error getParamValues. Missing value for not optional argument (' + i + ') = ' + name, node);
        var isTemplate = type === 'object' ? false : types.isTemplate(value);
        result.push({
            name: name.substr(0,1) === '&' ? name.substr(1) : name,
            value: value,
            type: type,
            isTemplate: isTemplate,
            isByRef: type === 'object'
        });
    }
    return result;
}
/*
    Helper class for IttfLoading evaluation passed to the esprima jswizzicontext.
*/
md.evalHelper = function (jsWizziContext, primaryModel) {
    this.jsWizziContext = jsWizziContext;
    this.jsWizziContext.set_ittf_evalHelper(this);
    this.loadContext = primaryModel.loadContext;
    this.textSep = work.textSep;
    if (typeof wizzi.runnerServer === 'object') {
        this.callApi = wizzi.runnerServer.callApi;
        this.loadWizziModel = wizzi.runnerServer.loadWizziModel;
    }
}
// set the IttfDocument local values context
md.evalHelper.prototype.s = function (modelKey) {
    this.jsWizziContext.set_IttfModelEvalContext(modelKey);
}
// set the node build-up context
md.evalHelper.prototype.n = function () {
    this.jsWizziContext.set_NodeContext();
}
// set the global values context
md.evalHelper.prototype.g = function () {
    this.jsWizziContext.set_GlobalContext();
}
// append a node to its parent node (basic build-up method)
md.evalHelper.prototype.a = function (parent, node, line) {
    if (typeof parent === 'undefined') {
        throw new Error("Error evalHelper. Parent undefined in add. At line " + line);
    }
    // why??? node.parent = node; // probably never used
    node.parent = parent; // from 2/12/2015
    if (typeof parent.childs === 'undefined') parent.childs = [];
    node.childs = [];
    parent.childs.push(node);
}
// interpolate a text string using the proper IttfDocument context values
md.evalHelper.prototype.ip = function (modelKey, templ, type, line) {
    // console.log('templ, this.ctx.values', templ, util.inspect(this.ctx.values, { depth: null }));
    var ret;
    try
    {
        var save = this.jsWizziContext.get_currentIttfModelKey();
        this.jsWizziContext.set_IttfModelEvalContext(modelKey, line);
        // console.log('Just before calling interpolate.values', util.inspect(this.jsWizziContext.values, { depth: 2 }));
        ret = interpolate(templ, this.jsWizziContext, { delimiter: '${}' });
        if (save === null) {
            this.jsWizziContext.set_NodeContext(modelKey);
        } else {
            this.jsWizziContext.set_IttfModelEvalContext(save);
        }
    } catch (ex) {
        var message = "evalHelper.ip()\n";
        message += '\nJust before throw, this.jsWizziContext.values:\n' + util.inspect(this.jsWizziContext.values, { depth: 2 });
        message += "\nwizzi-ittf.evalHelper.ip: " + templ + " modelKey " + modelKey + " type " + type + " at line " + line;
        if (!ex) {
            ex = new Error(message);
        }
        // wizzi.fail.warn(ex);
        // In case of --force proceed inside the try/catch flow
        throw ex;
    }
    return types.objectify(ret, type, { row: line});
}
// execute a runnerServer api call and return the result
md.evalHelper.prototype.api = function () {
    if (this.callApi) {
        return this.callApi.apply(wizzi.runnerServer, arguments);
    } else {
        var error = new wizzi.errors.IttfLoadError(
            'A wizzi.RunnerServer has not been started. You must create a wizzifile.js.',
            ''
        );
    }
}
// execute a runnerServer api call and return the result
md.evalHelper.prototype.model = function () {
    if (this.loadWizziModel) {
        return this.loadWizziModel.apply(wizzi.runnerServer, arguments);
    } else {
        var error = new wizzi.errors.IttfLoadError(
            'A wizzi.RunnerServer has not been started. You must create a wizzifile.js.',
            ''
        );
    }
}
/*
    Retrieve the value context of the ittfModel of name modelKey.
*/
md.evalHelper.prototype.getIttfModelEvalContext = function (modelKey, line) {
    // console.log('evalHelper.getIttfModelEvalContext.modelKey', modelKey);
    var ittfDocumentData = this.loadContext.getIttfModelData(modelKey);
    if (ittfDocumentData.evalContext) {
        // This re-evaluation is necessary for
        // . capturing newly declared global values ??? Why? are they not kept in the jswizzicontext global values ???
        // . re capturing object parameters (virtual byRefParams) that in the calling node are inside a loop (foreach)
        //   and are changed at each iteration
        //  Example
        //      $foreach item in items
        //          html-li( &item ) 
        // global values can be modified by child and mixed nodes
        // always refresh their values on the model scope values
        var ectx = ittfDocumentData.evalContext;
        var gv = this.jsWizziContext.getGlobalValues();
        for (var k in gv) {
            // console.log('getIttfModelEvalContext.global value ' + k + ' = ' + gv[k]);
            ectx[k] = gv[k];
        }
        if (ittfDocumentData.byRefParams) {
            // the ittfDocumentData has object parameters
            var self = this;
            ittfDocumentData.byRefParams.forEach(function (item) {
                var callingNode_ModelContext = self.getIttfModelEvalContext(
                    ittfDocumentData.ittfModel.$mixerModelKey
                    );
                /*
                log.info('evalHelper.getIttfModelEvalContext.$mixerModelKey, item.name',
                    ittfDocumentData.ittfModel.$mixerModelKey,
                    item,
                    ittfDocumentData.ittfDocPath,
                    line
                );
                */
                ectx[item] = callingNode_ModelContext[item];
                // console.log('evalHelper.callingNode_ModelContext[item.name]', callingNode_ModelContext[item]);
            });
        }
        return ectx;
    }
    // first use, create context, 
    // if $params set context with $params values
    var ectx = ittfDocumentData.evalContext = {};
    var gv = this.jsWizziContext.getGlobalValues();
    for (var k in gv) {
        // console.log('getIttfModelEvalContext.global value ' + k + ' = ' + gv[k]);
        ectx[k] = gv[k];
    }
    ectx['__dirname'] = path.dirname(ittfDocumentData.ittfDocPath);
    // console.log('getIttfModelEvalContext.$params', ittfDocumentData.ittfModel.$params, ittfDocumentData.ittfModel.$args);
    if (ittfDocumentData.ittfModel.$params) { // $params are copied from original source
        var args = ittfDocumentData.ittfModel.$args; // $args are set from the mixer ( are the calling node value )
        if (args && args.indexOf('${') > -1) { // $args may contain expressions
            // mixin call arguments must be interpolated
            // ittfDocumentData.ittfModel.$mixerModelKey is the modelKey of the calling node that mixed the ittfDocumentData.ittfModel
            // we must retrieve the calling node model context
            var callingNode_ModelContext = this.getIttfModelEvalContext(
                ittfDocumentData.ittfModel.$mixerModelKey
                );
            try {
                var interpolateArgsContext = new JsWizziContext();
                interpolateArgsContext.setValues(callingNode_ModelContext);
                args = interpolate(args, interpolateArgsContext, { delimiter: '${}' });
            } catch (ex) {
                ex.message += "Error interpolating args: " + ittfDocumentData.ittfModel.$args;
                wizzi.fail.warn(ex);
                // In case of --force proceed inside the try/catch flow
                throw ex;
            }
            // console.log('evalHelper.getIttfModelEvalContext.args', ittfDocumentData.ittfModel.$mixerModelKey, callingNode_ModelContext, ittfDocumentData.ittfModel.$args, args);
        }
        var paramValues = md.getParamValues(
            ittfDocumentData.ittfModel.$params,
            args,
            ittfDocumentData.ittfModel.nodes[0]
        );
        // console.log('getIttfModelEvalContext.paramValues', paramValues);
        for (var item, i = 0; i < paramValues.length; i++) {
            item = paramValues[i];
            if (item.isByRef) {
                // console.log('evalHelper.getIttfModelEvalContext.$mixerModelKey', ittfDocumentData.ittfModel.$mixerModelKey, item.name);
                var callingNode_ModelContext = this.getIttfModelEvalContext(
                    ittfDocumentData.ittfModel.$mixerModelKey
                );
                // console.log('evalHelper.callingNode_ModelContext[item.name]', callingNode_ModelContext[item.name]);
                ectx[item.name] = callingNode_ModelContext[item.name];
                ittfDocumentData.byRefParams = ittfDocumentData.byRefParams || [];
                ittfDocumentData.byRefParams.push(item.name);
            } else {
                ectx[item.name] = item.value;
            }
            // console.log('getIttfModelEvalContext.param value ' + item.name + ' = ' + item.value);
        }
    }
    // console.log('getIttfModelEvalContext.paramValues just before return', paramValues);
    return ectx;
}
