var util = require('util');
var path = require('path');
var wizzi = require('../wizzi');
var log = require('../util/log')(module);
var verify = wizzi.verify;
var jswizzirunner = require('../esprima/jswizzirunner');
var JsWizziContext = require('../esprima/jswizzicontext').context;
var dateUtil = require('../esprima/functions/dateutil');
//var ScriptCoder = require('../esprima/scriptcoder').ScriptCoder;
var ScriptCoder = require('../esprima/jsScriptCoder');
var helper = require('./helper');
module.exports = function (primaryModel, requestContext) {
    this.productionContext = requestContext.productionContext;
    // console.log('IttfEvaluator.ctor.modelContext', util.inspect(requestContext.modelContext, { depth: 1 }));
    // dataInfo object
    var data = { modelKey: null, counter: 0, startTime: dateUtil.now_GMYHMS() };
    
    // Create the IttfLoading evaluation script
    var jsScriptCoder = new ScriptCoder();
    jsScriptCoder.w('// ' + data.startTime);
    jsScriptCoder.w('var $0 = {};');
    primaryModel.nodes.forEach(function (item) {
        codify(item, 0, jsScriptCoder, data);
    });
    this.productionContext.addIttfModelBuildUpScript(
        primaryModel.uri, jsScriptCoder
        );
    // Create the jsWizziContext for global values.
    // It will be the parent context of the jsWizziContext passed to the jswizzirunner.
    var jsWizziGlobalContext = new JsWizziContext();
    // Initialize the jscontext global values with the modelContext object passed to this evaluator.
    jsWizziGlobalContext.setValues(requestContext.modelContext);
    var jsWizziContext = new JsWizziContext(jsWizziGlobalContext);
    // Set the $ function on the jsWizziContext.
    jsWizziContext.setValue('$', new helper.evalHelper(jsWizziContext, primaryModel));
    // Execute the build-up and evaluation of the Ittf model.
    try {
        // the BIG JOB
        jswizzirunner.run(jsScriptCoder.toCode(), jsWizziContext);
    } catch (ex) {
        // console.log('wizzi-core/ittf/lib/ittf/evaluator', ex);
        if (!ex) { ex = new Error();} // TODO ??? why ex can be undefined
        this.productionContext.raiseIttfEvaluationScriptError(primaryModel.uri, ex);
    }
    
    // retrieve result ( the final, re-built and evaluated Ittf model ) from jsWizzi jscontext values
    jsWizziContext.set_NodeContext();
    var n0 = jsWizziContext.values.$0;
    if (typeof n0 === 'undefined' || n0 == null) {
        throw new Error('No nodes returned after IttfEvaluation. Uri: ' + primaryModel.uri);
    }
    // on the primary ittfModel replace old nodes with evaluated nodes
    primaryModel.nodes = [];
    n0.childs.forEach(function (item) {
        item.parent = null;
        primaryModel.nodes.push(item);
    });
    // set statistic data
    primaryModel.data = {};
    primaryModel.data.createdAt = data.startTime;
    return primaryModel;
};
function codify(node, nparent, jsScriptCoder, data) {
    var nnode = ++data.counter, closeBlock = null;
    if (node.name == '$' || node.name == '$+') { // Template statement line
        setJsWizziContext(data, node.model.modelKey, jsScriptCoder);
        codeBlock(node, jsScriptCoder);
    } else if (node.name == '$global') { // 
        setJsWizziContext(data, 'global', jsScriptCoder);
        codeBlock(node, jsScriptCoder);
        setJsWizziContext(data, node.model.modelKey, jsScriptCoder);
    } else if (node.name == '$.') { // 
        // console.log("IttfEvaluator.node.value", node.value);
        setJsWizziContext(data, null, jsScriptCoder); // Is a node only operation, uses baseContext
        // TODO interpolate?
        var vparent = '$' + nparent;
        var value = codifyValue(node.model.modelKey, node.value, 'string', jsScriptCoder.length + 1);
        jsScriptCoder.w(vparent + ".v = " + vparent + ".v ? " + vparent + ".v : ''" + ' //' + node.id);
        jsScriptCoder.w(vparent + '.v += ($.textSep + ' + value + ');' + ' //' + node.id);
    } else if (node.name == '$if') {
        setJsWizziContext(data, node.model.modelKey, jsScriptCoder);
        jsScriptCoder.if(node.value, node);
        closeBlock = '}';
    } else if (node.name == '$else') {
        jsScriptCoder.else(node);
        data.modelKey = -1; // force set context on next statement
        closeBlock = '}';
    } else if (node.name == '$elif') {
        // No this raises an error. TODO (many tests here). setJsWizziContext(data, node.model.modelKey, jsScriptCoder);
        jsScriptCoder.elif(node.value, node);
        data.modelKey = -1; // force set context on next statement
        closeBlock = '}';
    } else if (node.name == '$while') {
        jsScriptCoder.while(node.value, node);
        data.modelKey = -1; // force set context on next statement
        closeBlock = '}';
    } else if (node.name == '$for') {
        setJsWizziContext(data, node.model.modelKey, jsScriptCoder);
        var items = node.value.split(' ');
        // TODO Verify format
        var namevalue = items[0].split(',');
        var name = namevalue[0];
        var value = namevalue[1];
        jsScriptCoder.w('var ' + name + ', ' + value + ';' + ' //' + node.id);
        jsScriptCoder.for(name + ' in ' + items[2] + ' //' + node.id);
        jsScriptCoder.w(value + ' = ' + items[2] + '[' + name + '];' + ' //' + node.id);
        closeBlock = '}';
    } else if (node.name == '$foreach') {
        setJsWizziContext(data, node.model.modelKey, jsScriptCoder);
        var items = node.value.split(' ');
        // TODO Verify format
        jsScriptCoder.for('var i' + nnode + '=0, l' + nnode + ' = ' + items[2] + '.length; i' + nnode + '<l' + nnode + '; i' + nnode + '++', node);
        jsScriptCoder.w('var ' + items[0] + ' = ' + items[2] + '[i' + nnode + '];' + ' //' + node.id);
        jsScriptCoder.w('var _index = i' + nnode + ';' + ' //' + node.id);
        jsScriptCoder.w('var _count = ' + items[2] + '.length;' + ' //' + node.id);
        closeBlock = '}';
    } else if (node.name == '$virtual') {
        ; // Do nothing, evaluate childs only
    } else {
        setJsWizziContext(data, null, jsScriptCoder);
        jsScriptCoder.w('var $' + nnode + ' = { ' +
            'n: "' + verify.escapename(node.name) + '", ' +
            (node.source ? 'source: ' + verify.escape(node.source) + ', ' : '') +
            'v: ' + codifyValue(node.model.modelKey, node.value, 'string', jsScriptCoder.length + 1) + ', ' +
            'r: ' + node.row + ', ' +
            'c: ' + node.col + ', ' +
            's: "' + node.model.modelKey + '", ' +
            'u: "' + node.sourceKey + '", ' +
            ' };' + ' //' + node.id);
        var vparent = '$' + nparent;
        jsScriptCoder.w('$.a(' + vparent + ', $' + nnode + ', ' + (jsScriptCoder.length + 1) + ');');
        nparent = nnode;
    }
    if (node.name !== '$' && node.name !== '$+') { // Template statement line
        node.childs.forEach(function (item) {
            codify(item, nparent, jsScriptCoder, data);
        });
        if (closeBlock) {
            jsScriptCoder.end();
            data.modelKey = -1; // force set context on next statement
        }
    }
}
function setJsWizziContext(data, modelKey, jsScriptCoder) {
    if (data.modelKey === modelKey) return;
    if (modelKey === null) {
        jsScriptCoder.w('$.n();');
    } else if (modelKey === 'global') {
        jsScriptCoder.w('$.g();');
    } else {
        jsScriptCoder.w('$.s("' + modelKey + '");');
    }
    data.modelKey = modelKey;
}
function codifyValue(modelKey, value, type, line) {
    if (typeof value === 'undefined' || value == null) return '""';
    if (type === 'string') {
        if (value.indexOf('${') > -1) return '$.ip("' + modelKey + '", ' + verify.escape(value) + ', "' + type + '", ' + line + ')';
        return verify.escape(value);
    }
    return value;
}
function codeBlock(node, jsScriptCoder) {
    if (node.name == '$' || node.name == '$+') { // Template statement line
        if (node.value && node.value.trim().length > 0) {
            jsScriptCoder.w(node.value);
        }
    } else {
        if (node.name && node.name.trim().length > 0) {
            jsScriptCoder.w(node.name + ' ' + (node.value || '') + ' //' + node.id);
        }
    }
    node.childs.forEach(function (item) {
        codeBlock(item, jsScriptCoder);
    });
}
