var util = require('util');
var path = require('path');
var wizzi = require('../wizzi');
var log = require('../util/log')(module, { tofile: true });
var verify = wizzi.verify;
var md = module.exports = {};
/*
    ModelStore generalizes the access to various 
    repository systems of IttfDocuments
        filesystem repository - implemented
        mongodb repository    - work in progress
    Is instantiated by the modelProvider
*/
var file = null;
var fsImpl = null;
var repoapi = {};
var repopath = null;
var repoImpl = null;
/*
    params
        ittfSource: Object {
            path: uri,
            storeKind: String ['repo'|'filesystem'], 
                repo means Mongodb repository (work in progress)
            storeDB: String
                the db connection string (to be implemented)
        }
*/
md.ModelStore = function (ittfSource) {
    if (ittfSource.storeKind === 'filesystem') {
        if (fsImpl === null) {
            file = require("wizzi-core").file;
            fsImpl = new FilesystemStore();
        }
        this.storeImpl = fsImpl;
    } else if (ittfSource.storeKind === 'repo') {
        throw new Error("Mongodb repo not implemented.");
        if (repoImpl === null) {
            repopath = require('../repo/util/repopath');
            require('../repo/api/document')(repoapi);
            repoImpl = new RepoStore();
        }
        this.storeImpl = repoImpl;
    } else {
        throw new Error('Error Ittf.ModelStore.ctor. Unknown storeKind: ' + ittfSource.storeKind);
    }
    this.storeKind = fsImpl.storeKind;
}
md.ModelStore.prototype.documentExists = function (filepath, callback) {
    this.storeImpl.documentExists(filepath, callback);
}
md.ModelStore.prototype.getModelContent = function (filepath, callback) {
    this.storeImpl.getModelContent(filepath, callback);
}
var FilesystemStore = function () {
    this.storeKind = 'filesystem';
}
FilesystemStore.prototype.documentExists = function (filepath, callback) {
    var documentExists;
    try {
        documentExists = file.isFile(filepath);
    } catch (ex) {
        var error = new wizzi.errors.IttfLoadError(ex.message + '\nChecking documentExists', filepath);
        return callback(error);
    }
    callback(null, documentExists);
}
FilesystemStore.prototype.getModelContent = function (filepath, callback) {
    var content;
    try
    {
        // sync call
        content = file.read(filepath);
    } catch (ex) {
        var error = new wizzi.errors.IttfLoadError(ex.message, filepath);
        return callback(error);
    }
    return callback(null, content);
}
var RepoStore = function () {
    this.storeKind = 'repo';
}
RepoStore.prototype.documentExists = function (documentPath, callback) {
    var repoPath = repopath(documentPath);
    repoPath.path = repoPath.path.substr(1);
    // log.info('RepoStore.documentExists.repoPath', repoPath);
    repoapi.getDocument(
        repoPath.userName,
        repoPath.projectName,
        repoPath.path,
        {},
        function (err, result) {
            if (err) {
                var error = new wizzi.errors.IttfLoadError(
                    util.inspect(err, { depth: null }) + '\nChecking documentExists',
                    filepath
                    );
                return callback(error);
            }
            // log.info('RepoStore.documentExists.result', result);
            callback(null, result != null);
        });
}
RepoStore.prototype.getModelContent = function (documentPath, callback) {
    var repoPath = repopath(documentPath);
    repoPath.path = repoPath.path.substr(1);
    // log.info('RepoStore.getModelContent.repoPath', repoPath);
    repoapi.readDocument(
        repoPath.userName,
        repoPath.projectName,
        repoPath.path,
        function (err, result) {
            if (err) {
                var error = new wizzi.errors.IttfLoadError(util.inspect(err, { depth: null }), filepath);
                return callback(error);
            }
            callback(null, result);
        }
    );
}
