var util = require('util');
var assert = require('assert');

var f_dateutil = require('./functions/dateutil');
var f_verify = require('./functions/verify');
var f_underscore = require('./functions/underscore');

function log(label, obj) {
    // console.log(label, util.inspect(obj, { depth: 2 }));
}

/*
    jswizzicontext is polimorphic. It may have one of three state:

    1) nodeContext               Context values of the build up of the final ittfModel tree structure.
    2) ittfModelContext          Context values in the scope of the current ittfModel.
    3) globalContext             Global context values visible to all ittfModel(s) 
                                 and to the ittfModel tree structure build up statements.
*/

var md = module.exports = {}

md.context = function(parent)
{
    this.parent = parent; // jswizzicontext for global values
    this.functions = {};
    this.values = {};
    this.declares = {};
    if (!parent) {
        for (k in defaultValues) {
            this.declare(k, defaultValues[k]);
        }
        for (k in knownCallables) {
            this.declare(k, knownCallables[k]);
        }
    }
}
md.context.prototype.declare = function (name, init) {
    this.declares[name] = true;
    if (typeof init !== 'undefined') {
        this.values[name] = init;
    }
}
md.context.prototype.undeclare = function (name, init) {
    if (this.declares[name] = true)
        delete this.declares[name];
    if (this.values[name] = true)
        delete this.values[name];
}
md.context.prototype.declFunction = function (name, node) {
    this.functions[name] = node;
}
md.context.prototype.setValue = function (name, value) {
    this.declare(name, value);
}
md.context.prototype.setValues = function (values) {
    values = values || {};
    for (k in values) {
        this.declare(k, values[k]);
    }
}
md.context.prototype.put = function (name, value) {
    if (this.declares[name] === true) {
        this.values[name] = value;
        return true;
    } else {
        var ok = this.parent ? this.parent.put(name, value) : false;
        if (ok === false) throw new Error('Not declared: ' + name);
    }
}
md.context.prototype.getValue = function (name) {
    if (this.declares[name] === true) {
        return this.values[name];
    } else if (this.parent) {
        return this.parent.get(name);
    } else {
        console.log('Esprima.jswizzicontext.get parent ', this.parent)
        console.log('Esprima.jswizzicontext.get declares ', this.declares)
        console.log('Esprima.jswizzicontext.get ittf_state ', this.ittf_state)
        console.log('Esprima.jswizzicontext.get currentIttfModelKey ', this.currentIttfModelKey)
        throw new Error('In esprima context. Not declared: ' + name);
    }
}
md.context.prototype.getGlobalValues = function () {
    if (this.parent) {
        return this.parent.values;
    } else {
        return this.values;
    }
}
md.context.prototype.getFunction = function (name) {
    return this.functions[name];
}
md.context.prototype.beginExpr = function () {
    this.exprPending = {};
}
md.context.prototype.endExpr = function () {
    for (k in this.exprPending) {
        this.put(k, this.exprPending[k]);
    }
}
md.context.prototype.push = function () {
    var ret = new md.context(this);
    return ret;
}
md.context.prototype.pop = function () {
    this.clear();
}
md.context.prototype.clear = function () {
    this.functions = {};
    this.values = {};
    this.declares = {};
    if (!this.parent) {
        for (k in defaultValues) {
            this.declare(k, defaultValues[k]);
        }
        for (k in knownCallables) {
            this.declare(k, knownCallables[k]);
        }
    }
}

/*
    Set the evalHelper before starting the jsrunner
*/
md.context.prototype.set_ittf_evalHelper = function (evalHelper) {
    // this jswizzicontext has already been ctor injected into the evalHelper
    this.evalHelper = evalHelper;
    this.ittf_state = ittfStates.nodeContext; // starting jswizzicontext state
    this.currentIttfModelKey = null;
}

/*
    set the state of this jswizzicontext to nodeContext.
*/
md.context.prototype.set_NodeContext = function () {
    // console.log('set_NodeContext', this.ittf_state, ittfStates.nodeContext);
    if (this.ittf_state === ittfStates.nodeContext) return;

    if (this.nodeContextValues) {
        // set the values of this jswizzicontext to the nodeContext values
        this.declares = this.nodeContextDeclares;
        this.values = this.nodeContextValues;
    } else {
        throw new Error('wizzi-ittf.esprima.jswizzicontext.ittf_set_NodeContext Sequence error: this.nodeContextValues must be an object.');
    }

    // set the state of the context
    this.ittf_state = ittfStates.nodeContext;
    this.currentIttfModelKey = null;
}

/*
    set the state of this jswizzicontext to globalContext.
*/
md.context.prototype.set_GlobalContext = function () {
    // console.log('set_GlobalContext', this.ittf_state, ittfStates.globalContext);
    if (this.ittf_state === ittfStates.globalContext) return;

    if (this.ittf_state === ittfStates.nodeContext && !this.nodeContextValues) {
        // save the node context values
        this.nodeContextDeclares = this.declares;
        this.nodeContextValues = this.values;
    }

    // The global context values are the parent values of this jswizzicontext.
    this.declares = this.parent.declares;
    this.values = this.parent.values;

    // do not break the command chain 
    this.setValue('$', this.evalHelper);

    // set the state of the context
    this.ittf_state = ittfStates.globalContext;
    this.currentIttfModelKey = null;
}
/*
    Transforms this in the value context of the ittfModel of key 'modelKey'
*/
md.context.prototype.set_IttfModelEvalContext = function (modelKey, line) {
    // console.log('set_IttfModelEvalContext', this.currentIttfModelKey, modelKey);
    if (this.currentIttfModelKey === modelKey) return;

    if (this.ittf_state === ittfStates.nodeContext && !this.nodeContextValues) {
        // save the node context values
        this.nodeContextDeclares = this.declares;
        this.nodeContextValues = this.values;
    }

    // retrieve the context of the ittfModel of key 'modelKey'
    var modelContext = this.evalHelper.getIttfModelEvalContext(modelKey, line);

    // set the values of 'this' esprima context
    modelContext = modelContext || {};
    this.declares = {};
    for (k in modelContext) {
        // console.log('ittf_set_IttfModelEvalContext.decl', modelKey, k /*, modelContext[k] */);
        this.declare(k, modelContext[k]);
    }

    this.values = modelContext;

    // do not break the command chain
    this.setValue('$', this.evalHelper);

    // set the state of the context
    this.ittf_state = ittfStates.modelContext;
    this.currentIttfModelKey = modelKey;
}
md.context.prototype.get_currentIttfModelKey = function () {
    return this.currentIttfModelKey;
}

var defaultValues = {
    '__dollardollar': '$$',
    '__dollaraster': '$*',
    '__asterdollar': '*$'
};

var knownCallables = {
    'console': console,
    '_': f_underscore,
    'Object': Object,
    'Math': Math,
    'path': require('path'),
    'dateUtil': f_dateutil,
    'verify': f_verify
};

var ittfStates = {
    nodeContext: 0,  // ittf tree nodes building context
    modelContext: 1,  // single model document context
    globalContext: 2   // global context (in the parent context)
};

