/*
    The ittf model load context
*/

var util = require('util');
var path = require('path');
var wizzi = require('../wizzi');
var log = require('../util/log')(module, { tofile: true });
var verify = wizzi.verify;

// TODO
// describe source keys and model keys

/*
    loadContext : {
        ittfSources: Array,
        ittfDocumentDatas: Array, // of IttfModel
        sourceCount: Integer, 
        modelCount: Integer
        nodeCount: Integer
    }
    
    ittfDocumentData : {
        ittfDocumentUri: String, // uri
        schema: String,
        sourceKey: sourceKey,
        modelKey: modelKey,
        ittfModel: ittfModelCloned,
        // ----
        evalContext: Object, //
        byRefParams: Array, // of String

    }
 */

function loadContext() {
    this.ittfSources = {};
    this.ittfDocumentDatas = {};
    this.sourceCount = 0;
    this.modelCount = 0;
    this.nodeCount = 0;
}

/*
    The same IttfDocument can be mixed many times inside a Wizzi model.
    The ittfDocumentDatas collection stores a cloned instance of the loaded IttfModel,
    that is specific to a mixed document.
    Parameters passed to a document and variables declared in the document have
    document scope, so each cloned IttfModel has its local value context that will
    be associated to it (through the modelKey).
*/
loadContext.prototype.addIttfModel = function (ittfDocumentUri, schema, ittfModelCloned, options) {
    // console.log('loadContext.addIttfModel.ittfDocumentUri, schema', ittfDocumentUri, schema);
    var sourceKey = this.getSourceKey(ittfDocumentUri);
    // console.log('ittf.loadContext.addIttfModel.ittfDocumentUri, sourceKey', ittfDocumentUri, sourceKey);
    if (options.include) {
        return {
            sourceKey: sourceKey,
            modelKey: options.includerModelKey,
        }
    } 
    var modelKey = 'f' + (++this.modelCount);
    var modelData = {
        ittfDocumentUri: ittfDocumentUri,
        schema: schema,
        sourceKey: sourceKey,
        modelKey: modelKey,
        ittfModel: ittfModelCloned
    };
    this.ittfDocumentDatas[modelKey] = modelData;
    return modelData;
}

loadContext.prototype.getIttfModelData = function (modelKey) {
    return this.ittfDocumentDatas[modelKey];
}

loadContext.prototype.addIttfDocument = function (ittfDocumentUri, ittfSourceContent) {
    var sourceKey = 'f' + (++this.sourceCount);
    var sourceData = {
        ittfDocumentUri: ittfDocumentUri,
        sourceKey: sourceKey,
        content: ittfSourceContent
    };
    this.ittfSources[sourceKey] = sourceData;
    return sourceData;
}

loadContext.prototype.getIttfDocumentUri = function (sourceKey) {
    var sourceData = this.ittfSources[sourceKey];
    return sourceData ? sourceData.ittfDocumentUri : 'Source ittfDocumentUri unknown';
}

loadContext.prototype.getSourceKey = function (ittfDocumentUri) {
    for (var k in this.ittfSources) {
        if (this.ittfSources[k].ittfDocumentUri === ittfDocumentUri) {
            return k;
        }
    }
    return null;
}

loadContext.prototype.getIttfDocuments = function () {
    var ret = {};
    for (var k in this.ittfSources) {
        var ittfSource = this.ittfSources[k]
        ret[this.getIttfDocumentUri(ittfSource.sourceKey)] = { sourceKey: ittfSource.sourceKey };
    }
    return ret;
}

loadContext.prototype.getNewNodeId = function () {
    return ++this.nodeCount;
}

module.exports = loadContext;

