var util = require('util');
var wizzi = require('../wizzi');

var md = module.exports = {};

function NodeError (message, node) {
    var msg = [message];
    if (node) {
        if (node.WmtSourceLineInfo) {
            var info = node.WmtSourceLineInfo;
            msg.push(' at row: ' + info.row);
            msg.push(', col: ' + info.col);
            msg.push(', source: ' + info.source);
            msg.push(', in file: ' + 'TODO'); // util.node.getSource(info)
        } else if (node.wzSourceLineInfo) {
            var info = node.wzSourceLineInfo;
            var filePath = 'TODO';
            if (node.wzSourceFilepath) {
                filePath = node.wzSourceFilepath(info.sourceKey);
            }
            // console.log('util.error.info', info);
            msg.push(' at row: ' + info.row);
            msg.push(', col: ' + info.col);
            msg.push(', source: ' + info.sourceKey);
            msg.push(', in file: ' + filePath); // util.node.getSource(info)
        } else if (node.row) {
            msg.push(' at row: ' + node.row);
            msg.push(', col: ' + node.col);
        }
    }
    this.message = msg.join('');
    this.node = node;
}
NodeError.prototype.toString = function () {
    return this.message;
}
NodeError.prototype = Object.create(Error.prototype);
NodeError.prototype.constructor = NodeError;

function NotFoundError(resourceType, name, srcPath) {
    this.name = 'NotFoundError';
    this.message = resourceType + ': ' + name + ' not found, processing document ' + getSrcPathInfo(srcPath);
    this.stack = (new Error()).stack;
}
NotFoundError.prototype = Object.create(Error.prototype);
NotFoundError.prototype.constructor = NotFoundError;

function IttfLoadError(message, srcPath, node, ex) {
    if (!ex) { ex = node; node = null; };
    this.name = 'IttfLoadError';
    this.message = 'Error: ' + message + '\nLoading ittf document ' + getSrcPathInfo(srcPath);
    if (node) {
        /*
        console.log(
            'IttfLoadError.node',
            util.inspect(node, { depth: 2 })
        );
        */
        this.message += '\n' + new NodeError('', node).message;
    }
    this.stack = ex && ex.stack ? ex.stack : (new Error()).stack;
}
IttfLoadError.prototype = Object.create(Error.prototype);
IttfLoadError.prototype.constructor = IttfLoadError;

function WizziModelLoadError(message, srcPath, ex) {
    this.name = 'WizziModelLoadError';
    this.message = 'Error: ' + message + '\nLoading ittf document ' + getSrcPathInfo(srcPath);
    this.stack = ex && ex.stack ? ex.stack : (new Error()).stack;
}
WizziModelLoadError.prototype = Object.create(Error.prototype);
WizziModelLoadError.prototype.constructor = WizziModelLoadError;

function ModelTransformationError(message, generation, srcPath, ex) {
    this.name = 'ModelTransformationError';
    this.message = 'Error: ' + message + '\nDuring generation:' + generation + ', processing document ' + getSrcPathInfo(srcPath);
    this.stack = ex && ex.stack ? ex.stack : (new Error()).stack;
}
ModelTransformationError.prototype = Object.create(Error.prototype);
ModelTransformationError.prototype.constructor = ModelTransformationError;

function ArtifactGenerationError(message, generation, srcPath, ex) {
    this.name = 'ArtifactGenerationError';
    this.message = 'Error: ' + message + '\nDuring generation:' + generation + ', processing document ' + getSrcPathInfo(srcPath);
    this.stack = ex && ex.stack ? ex.stack : (new Error()).stack;
}
ArtifactGenerationError.prototype = Object.create(Error.prototype);
ArtifactGenerationError.prototype.constructor = ArtifactGenerationError;

function WizziInvalidRequestError(message, srcPath, ex) {
    this.name = 'WizziInvalidRequestError';
    this.message = message;
    this.stack = ex && ex.stack ? ex.stack : (new Error()).stack;
}
WizziInvalidRequestError.prototype = Object.create(Error.prototype);
WizziInvalidRequestError.prototype.constructor = WizziInvalidRequestError;

md.artifactGenerationError = function (message, generation, node) {
    var error = new wizzi.errors.ArtifactGenerationError(
        message,
        generation,
        node // srcPath object
        );
    wizzi.fail.warn(error);
    throw error;
}

md.NodeError = NodeError;
md.NotFoundError = NotFoundError;
md.IttfLoadError = IttfLoadError;
md.WizziModelLoadError = WizziModelLoadError;
md.ModelTransformationError = ModelTransformationError;
md.ArtifactGenerationError = ArtifactGenerationError;
md.WizziInvalidRequestError = WizziInvalidRequestError;

// TODO
md.error = function (message, node) {
    var exception = new md.__NodeError(message, node);
    wizzi.fail.warn(exception);
    // In case of --force proceed inside the try/catch flow
    throw exception;
}

function getSrcPathInfo(srcPath) {
    if (typeof srcPath === 'string') return srcPath;
    if (srcPath && typeof srcPath === 'object') {
        var msg = getSrcPathInfoFromNode(srcPath);
        return msg ? msg : srcPath.toString();
    }
    return 'unavailable';
}

function getSrcPathInfoFromNode(node) {
    var msg = [];
    if (node) {
        if (node.WmtSourceLineInfo) {
            var info = node.WmtSourceLineInfo;
            msg.push(' at row: ' + info.row);
            msg.push(', col: ' + info.col);
            msg.push(', source: ' + info.source);
            msg.push(', in file: ' + 'TODO'); // util.node.getSource(info)
        } else if (node.wzSourceLineInfo) {
            var info = node.wzSourceLineInfo;
            var filePath = 'TODO';
            if (node.wzSourceFilepath) {
                filePath = node.wzSourceFilepath(info.sourceKey);
            }
            // console.log('util.error.info', info);
            msg.push(' at row: ' + info.row);
            msg.push(', col: ' + info.col);
            msg.push(', source: ' + info.sourceKey);
            msg.push(', in file: ' + filePath); // util.node.getSource(info)
        }
    }
    return msg.length == 0 ? null : msg.join('');
}


