// generator: wizzi-lab-artifatcs/lib/js/module/gen/main.js, utc time: Thu, 16 Feb 2017 09:35:15 GMT
var path = require('path');
var log = require('../util/log')(module);
var verify = require('../util/verify');
var file = require('../util/file');
var fail = require('../util/fail');
var errors = require('../util/errors');
// TODO
// Explain how Wizzi model types and Artifact generators are retrieved
// Repository class
Repository = function(options) {
    this.options = options;
    this.wizziFactoryPackages = [];
}
Repository.prototype.registerWizziFactoryPackage = function(wizziFactoryPackage) {
    this.wizziFactoryPackages.push(wizziFactoryPackage);
}
Repository.prototype.getWizziModelFactory = function(schemaName) {
    // console.log('Repository.getWizziModelFactory.schemaName: ' + schemaName);
    var found, foundInModule = null, result = null;
    // all wizziFactoryPackages are searched, last encountered results override the previous.
    var i, i_len=this.wizziFactoryPackages.length, item;
    for (i=0; i<i_len; i++) {
        item = this.wizziFactoryPackages[i];
        // log 'searching model loader', schemaName, ' in module', item.name
        found = null;
        if (verify.isFunction(item[schemaName])) {
            found = item[schemaName];
        }
        else if (verify.isFunction(item['getWizziModelFactory'])) {
            try {
                var m = item['getWizziModelFactory'](schemaName);
                if (m && verify.isFunction(m.load)) {
                    found = m.load;
                }
            } catch (ex) {
                var msg = 'searching schemaName: ' + schemaName + ', in wizziFactoryPackage: ' + item.name + ', message ' + ex.message;
                ex.message = 'getWizziModelFactory. Exception: ' + msg;
                fail.warn(ex);
                found = null;
            }
        }
        if (found) {
            result = found;
            foundInModule = item.name;
        }
    }
    if (!result) {
        log.error('getWizziModelFactory. Cannot find model loader: ' + schemaName + '\n' + 'availables: ' + availables.join(', '));
    }
    else {
        log.success('getWizziModelFactory. Found model loader: ' + schemaName + ' in module ' + foundInModule);
    }
    return result;
}
Repository.prototype.getWizziSchemaObject = function(schemaName) {
    // console.log('Repository.getWizziSchemaObject.schemaName: ' + schemaName);
    var found, foundInModule = null, result = null;
    // all wizziFactoryPackages are searched, last encountered results override the previous.
    var i, i_len=this.wizziFactoryPackages.length, item;
    for (i=0; i<i_len; i++) {
        item = this.wizziFactoryPackages[i];
        // log 'searching wizzi schema object', schemaName, ' in module', item.name
        found = null;
        if (verify.isFunction(item[schemaName])) {
            found = item[schemaName];
        }
        else if (verify.isFunction(item['getWizziSchemaObject'])) {
            try {
                var m = item['getWizziSchemaObject'](schemaName);
                if (m) {
                    found = m;
                }
            } catch (ex) {
                var msg = 'searching schemaName: ' + schemaName + ', in wizziFactoryPackage: ' + item.name + ', message ' + ex.message;
                ex.message = 'getWizziSchemaObject. Exception: ' + msg;
                fail.warn(ex);
                found = null;
            }
        }
        if (found) {
            result = found;
            foundInModule = item.name;
        }
    }
    if (!result) {
        log.error('getWizziSchemaObject. Cannot find schema object: ' + schemaName + '\n' + 'availables: ' + availables.join(', '));
    }
    else {
        log.success('getWizziSchemaObject. Found schema object: ' + schemaName + ' in module ' + foundInModule);
    }
    return result;
}
Repository.prototype.getModelTransformer = function(transformName) {
    // log 'Repository.getModelTransformer.transformName: ' + transformName
    var found, foundInModule = null, result = null;
    // all wizziFactoryPackages are searched, last encountered results override the previous.
    var i, i_len=this.wizziFactoryPackages.length, item;
    for (i=0; i<i_len; i++) {
        item = this.wizziFactoryPackages[i];
        // log 'searching transformer ', transformName, ' in module', item.name
        found = null;
        if (verify.isFunction(item['getModelTransformer'])) {
            try {
                var m = item['getModelTransformer'](transformName);
                if (m && verify.isFunction(m.trans)) {
                    found = m.trans;
                }
            } catch (ex) {
                var msg = 'searching transformName: ' + transformName + ', in wizziFactoryPackage: ' + item.name + ', message ' + ex.message;
                ex.message = 'getModelTransformer. Exception: ' + msg;
                fail.warn(ex);
                found = null;
            }
        }
        if (found) {
            result = found;
            foundInModule = item.name;
        }
    }
    if (!result) {
        log.error('getModelTransformer. Cannot find transformer: ' + transformName);
    }
    else {
        log.success('getModelTransformer. Found transformer: ' + transformName + ' in module ' + foundInModule);
    }
    return result;
}
Repository.prototype.getArtifactGenerator = function(artifactName) {
    // log 'Repository.getArtifactGenerator.artifactName: ' + artifactName
    var found, foundInModule = null, result = null;
    // all wizziFactoryPackages are searched, last encountered results override the previous.
    var i, i_len=this.wizziFactoryPackages.length, item;
    for (i=0; i<i_len; i++) {
        item = this.wizziFactoryPackages[i];
        // log 'searching artifact ', artifactName, ' in module', item.name
        found = null;
        if (verify.isFunction(item['getArtifactGenerator'])) {
            try {
                found = item['getArtifactGenerator'](artifactName);
            } catch (ex) {
                var msg = 'searching artifactName: ' + artifactName + ', in wizziFactoryPackage: ' + item.name + ', message ' + ex.message;
                ex.message = 'getArtifactGenerator. Exception: ' + msg;
                fail.warn(ex);
                found = null;
            }
        }
        if (found) {
            result = found;
            foundInModule = item.name;
        }
    }
    if (!result) {
        log.error('getArtifactGenerator. Cannot find artifact generator: ' + artifactName);
    }
    else {
        log.success('getArtifactGenerator. Found artifact generator: ' + artifactName + ' in module ' + foundInModule);
    }
    return result;
}
Repository.prototype.loadWizziModel = function(wizzischema, ittfDocumentUri, context, callback) {
    var factory = this.getWizziModelFactory(wizzischema);
    if (!factory) {
        var error = new errors.NotFoundError('WizziModelFactory', wizzischema, ittfDocumentUri);
        return callback(error);
    }
    context = context || {};
    // log 'factory', factory
    factory(ittfDocumentUri, context, function(err, wizziModel) {
        if (err) {
            return callback(err);
        }
        callback(null, formatWizziModel(wizziModel, context.__formatOptions));
    });
}
Repository.prototype.transformModel = function(transformName, model, context, callback) {
    var trans = this.getModelTransformer(transformName);
    if (!trans) {
        var error = new errors.NotFoundError('ModelTransformer', transformName, 'wizzi-core repository');
        return callback(error);
    }
    context = context || {};
    // log 'trans', trans
    trans(model, context, function(err, modelTransformed) {
        if (err) {
            return callback(err);
        }
        callback(null, modelTransformed);
    });
}
Repository.prototype.loadWizziSchema = function(wfpackageuri, wizzischema, formatOptions, callback) {
    if (!formatOptions || formatOptions.format !== 'json') {
        var error = new errors.WizziInvalidRequestError('loadWizziSchema invalid format');
        return callback(error);
    }
    var schemaJsonUri = path.join(wfpackageuri, '/lib/wizzi/models/' + wizzischema + '-schema.g.json');
    if (!file.isFile(schemaJsonUri)) {
        var error = new errors.NotFoundError('WizziSchema', wizzischema, schemaJsonUri);
        return callback(error);
    }
    try {
        var schemajson = file.readJSON(schemaJsonUri);
        return callback(null, schemajson);
    } catch (ex) {
        return callback(ex.message);
    }
}

function formatWizziModel(wizziModel, formatOptions) {
    if (!formatOptions) {
        return wizziModel;
    }
    var partial = false;
    var ret = {};
    if (formatOptions.ittfSources) {
        partial = true;
        ret.loadContext = {};
        ret.loadContext.ittfSources = wizziModel.loadContext.ittfSources;
    }
    if (partial) {
        return ret;
    }
    else {
        return wizziModel;
    }
}
module.exports = Repository;
